@@ -1,14 +1,14 @@
 #' Group input by rows
 #'
-#' \code{rowwise} is used for the results of \code{\link{do}} when you
+#' `rowwise()` is used for the results of [do()] when you
 #' create list-variables. It is also useful to support arbitrary
 #' complex operations that need to be applied to each row.
 #'
-#' Currently \code{rowwise} grouping only works with data frames. Its
+#' Currently, rowwise grouping only works with data frames. Its
 #' main impact is to allow you to work with list-variables in
-#' \code{\link{summarise}} and \code{\link{mutate}} without having to
-#' use \code{[[1]]}. This makes \code{summarise()} on a rowwise tbl
-#' effectively equivalent to plyr's \code{ldply}.
+#' [summarise()] and [mutate()] without having to
+#' use \code{[[1]]}. This makes `summarise()` on a rowwise tbl
+#' effectively equivalent to [plyr::ldply()].
 #'
 #' @param data Input data frame.
 #' @export
@@ -19,9 +19,12 @@
 rowwise <- function(data) {
   stopifnot(is.data.frame(data))
 
-  structure(data, class = c("rowwise_df", "tbl_df", "data.frame"))
+  assert_all_white_list(data)
+  structure(data, class = c("rowwise_df", "tbl_df", "tbl", "data.frame"))
 }
 
+setOldClass(c("rowwise_df", "tbl_df", "tbl", "data.frame"))
+
 #' @export
 print.rowwise_df <- function(x, ..., n = NULL, width = NULL) {
   cat("Source: local data frame ", dim_desc(x), "\n", sep = "")
@@ -32,8 +35,8 @@ print.rowwise_df <- function(x, ..., n = NULL, width = NULL) {
 }
 
 #' @export
-ungroup.rowwise_df <- function(x) {
-  class(x) <- c( "tbl_df", "data.frame")
+ungroup.rowwise_df <- function(x, ...) {
+  class(x) <- c("tbl_df", "tbl", "data.frame")
   x
 }
 #' @export
@@ -53,33 +56,40 @@ n_groups.rowwise_df <- function(x) {
 }
 
 #' @export
-group_by_.rowwise_df <- function(.data, ..., .dots, add = FALSE) {
-  warning("Grouping rowwise data frame strips rowwise nature", call. = FALSE)
+group_by.rowwise_df <- function(.data, ..., add = FALSE) {
+  warn("Grouping rowwise data frame strips rowwise nature")
   .data <- ungroup(.data)
 
-  groups <- group_by_prepare(.data, ..., .dots = .dots, add = add)
-  grouped_df(groups$data, groups$groups)
+  groups <- group_by_prepare(.data, ..., add = add)
+  grouped_df(groups$data, groups$group_names)
+}
+#' @export
+group_by_.rowwise_df <- function(.data, ..., .dots = list(), add = FALSE) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  group_by(.data, !!! dots, add = add)
 }
 
 
 # Do ---------------------------------------------------------------------------
 
 #' @export
-do_.rowwise_df <- function(.data, ..., .dots) {
+do.rowwise_df <- function(.data, ...) {
   # Create ungroup version of data frame suitable for subsetting
   group_data <- ungroup(.data)
-
-  args <- lazyeval::all_dots(.dots, ...)
-  named <- named_args(args)
-  env <- new.env(parent = lazyeval::common_env(args))
   index <- attr(.data, "indices")
 
+  args <- quos(...)
+  named <- named_args(args)
+
   # Create new environment, inheriting from parent, with an active binding
   # for . that resolves to the current subset. `_i` is found in environment
   # of this function because of usual scoping rules.
-  makeActiveBinding(".", function() {
-    lapply(group_data[`_i`, , drop = FALSE], "[[", 1)
-  }, env)
+  env <- child_env(NULL)
+  current_row <- function() lapply(group_data[`_i`, , drop = FALSE], "[[", 1)
+  env_bind_fns(.env = env, . = current_row, .data = current_row)
+
+  overscope <- new_overscope(env)
+  on.exit(overscope_clean(overscope))
 
   n <- nrow(.data)
   m <- length(args)
@@ -90,7 +100,7 @@ do_.rowwise_df <- function(.data, ..., .dots) {
 
   for (`_i` in seq_len(n)) {
     for (j in seq_len(m)) {
-      out[[j]][`_i`] <- list(eval(args[[j]]$expr, envir = env))
+      out[[j]][`_i`] <- list(overscope_eval_next(overscope, args[[j]]))
       p$tick()$print()
     }
   }
@@ -101,3 +111,8 @@ do_.rowwise_df <- function(.data, ..., .dots) {
     label_output_list(NULL, out, groups(.data))
   }
 }
+#' @export
+do_.rowwise_df <- function(.data, ..., .dots = list()) {
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  do(.data, !!! dots)
+}
